// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __CONNECTION_H
#define __CONNECTION_H

#include "source.h"
#include "tunermanager.h"
#include "stringrep.h"

#include <string>

class InputBuffer;
class OutputBuffer;
class Scheduler;

class Connection : public Sink, public Selectable, public StringRep {
public:
  Connection(TunerManager *tm, int dataFD);
  virtual ~Connection();

  virtual void addSelectFDs(Select &) const;
  virtual bool isReady(const Select &) const;

  virtual bool processReady(const Select &s);

  virtual void receiveData(unsigned char *data, unsigned size);

  virtual void notifyUnsubscribe(Source *s);
  virtual int getPriority() const;
  virtual void setPriority(int p);

  bool canTune(const std::string &newType, const std::string &newChannel, int p) const;
  bool tune(const std::string &newType, const std::string &newChannel, bool convertToPS, int p);

  bool getInterrupted() const;

private:
  bool doSubscribe();
  void doUnsubscribe();

protected:
  bool isValidType(const std::string &type) const;
  bool isValidChannel(const std::string &type, const std::string &channel) const;
  void getTunableChannels(const std::string &type, int priority, StringList &channels) const;

  bool isRetuning() const;
  bool numDemuxers() const;

  void setRemoveMe(bool status);

  time_t getCurrentTime() const;

  virtual std::string getType() const = 0;
  virtual std::ostream& printOn(std::ostream &) const;

private:
  int dataFD;
  int priority;
  bool tuned;
  bool retuning;
  TunerManager *tm;
  std::string channel, type;
  OutputBuffer *dataOutput;
  SourceList demuxers;
  bool removeMe;		/** kill this connection */
  bool interrupted;		/** interrupted by higher priority recording */
  bool lostPacket;		/** client is not keeping up */
  bool convertToPS;		/** convert to program stream */
  time_t lastTime;		/** Last time printed buffer stats */
};


#endif // __CONNECTION_H
