/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "config.h"
#include "dvbd.h"
#include "tunermanager.h"
#include "connectionmanager.h"
#include "connection.h"
#include "lobby.h"
#include "configfile.h"
#include "stringutil.h"
#include "scheduler.h"

#include <iostream>

int debugLevel;

int main(int argc, char *argv[])
{
  // improve iostream performance
  std::ios::sync_with_stdio(false);

  ConfigFile config;
  
  config.set("socket", DEFAULT_SOCKET_FILE);
  config.set("socket-uid", DEFAULT_SOCKET_UID);
  config.set("socket-gid", DEFAULT_SOCKET_GID);
  config.set("socket-mode", DEFAULT_SOCKET_MODE);
  config.set("pid", DEFAULT_PID_FILE);
  config.set("log", DEFAULT_LOG_FILE);
  config.set("schedule", DEFAULT_SCHEDULE_FILE);
  config.set("numcards", "1");
  config.set("card0", DEFAULT_CARD);
  config.set("debug", "0");

  std::string configFilename;

  if (argc >= 2) {
    configFilename = argv[1];
    if (!config.read(configFilename)) {
      std::cerr << "Fatal error: failed to open config file: " 
		<< configFilename << std::endl;
      return 1;
    }
  }
  else {
    configFilename = DEFAULT_CONFIG_FILE;
    config.read(configFilename);
  }

  debugLevel = toInt(config.get("debug"));

  if (debugLevel == 0)
    if (!daemonize(config))
      return 1;

  TunerManager tm;
  if (!tm.parseConfig(config)) {
    std::cerr << "Fatal error: failed to parse cards defined in configuration file"
	      << std::endl;
    return 1;
  }

  ConnectionManager cm;

  Scheduler scheduler(config.get("schedule"), &tm, &cm);

  std::string socketFilename = config.get("socket");
  std::string socketUid = config.get("socket-uid");
  std::string socketGid = config.get("socket-gid");
  std::string socketMode = config.get("socket-mode");
  Lobby lobby(socketFilename, socketUid, socketGid, socketMode, &tm, &cm, &scheduler);

  setupSignalHandlers();

  Select s;
  do {
    s.clearFDs();

    lobby.addSelectFDs(s);
    tm.addSelectFDs(s);
    cm.addSelectFDs(s);
    scheduler.addSelectFDs(s);

    s.wait();

    scheduler.process();
    lobby.processReady(s);
    cm.processReady(s);
    tm.processReady(s);

  } while (true);
  return 0;
}
