/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "inputbuffer.h"
#include <unistd.h>
#include <cerrno>

InputBuffer::InputBuffer(int fd, int initSize, int maxSize)
  : CircularBuffer(initSize, maxSize), fd(fd)
{
}

InputBuffer::~InputBuffer()
{
}

void InputBuffer::addSelectFDs(Select &s) const
{
  s.addReadFD(fd);
}

bool InputBuffer::isReady(const Select &s) const
{
  return s.readyToRead(fd);
}

bool InputBuffer::process()
{
  int spaceAvailable = getMaxSize() - getUsed();

  if (spaceAvailable == 0) {
    setOverflowed();
    return true;
  }

  char data[spaceAvailable];
  int bytesRead;

  bytesRead = ::read(fd, data, spaceAvailable);
  if (bytesRead <= 0)
    return errno == EAGAIN;

  write(data, bytesRead);
  return true;
}
