/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "config.h"
#include "utils.h"
#include "stringutil.h"
#include "recording.h"

Recording::~Recording()
{
}

std::ostream& Recording::printOn(std::ostream &o) const
{
  char dateTime[128];

  strftime(dateTime, sizeof(dateTime), "%Y-%m-%d %H:%M:%S", 
	   localtime(&startTime));

  o << job << " " << uid << " " << dateTime << " " << type << " " 
    << escapeWS(channel) << " " << ::toString(convertToPS) << " "
    << duration << " " << priority << " " << escapeWS(path) << " "
    << "[" << escapeWS(extra) << "]";

  return o;
}

std::istream &operator >> (std::istream &i, Recording &r)
{
  std::string datePart, timePart, channel, path, convertToPS, extra;

  i >> r.job 
    >> r.uid >> datePart >> timePart >> r.type >> channel
    >> convertToPS >> r.duration >> r.priority >> path
    >> extra;

  r.channel = unescapeWS(channel);
  r.convertToPS = toBool(convertToPS);
  r.path = unescapeWS(path);

  if (extra.size() >= 2)
    r.extra = unescapeWS(extra.substr(1, extra.size() - 2));
  else
    r.extra = "";
  
  std::string dateTime = datePart + " " + timePart;
  struct tm tm;
  if (strptime(dateTime.c_str(), "%Y-%m-%d %H:%M:%S", &tm) != NULL)
    r.startTime = mktime(&tm);

  return i;
}

